#include <windows.h> 
#include <formatio.h>
#include <cvirte.h>		
#include <userint.h>
#include <FTD2XX.h>
#include <ansi_c.h>
#include "iClock_WiFi.h"    

// ########## DEFINITIONEN ##########  

#define EE_EVENT_LOG_SIZE		100

// ########## DEKLARATIONEN ##########  

void FTDI_Get_Devices (void);
void FTDI_Open_Device (int iDevNum);
void FTDI_Close_Device (void);
void FTDI_Command (int iReadLen, char *sSerReadBuffer, char *sSerWriteBuffer);  
void FTDI_Purge (void); 
void Get_Device_Data (void);
void Remove_CR (char *sString);
void String_To_IP (char *sIP, int *iIP3, int *iIP2, int *iIP1, int *iIP0); 
void Network_Scan (void);

// ########## GLOBALE VARIABLEN ##########

static int hdMainPanel;						// Handle vom Main Panel
static int hdTab0;   
static int hdTab1;   
static int hdTab2;
static int hdDeviPanel;						// Handle vom Device Panel
static int hdNetwPanel;						// Handle vom Network Panel
static int iButtonDeviPanel = -1;			// Knopf im Device Panel gedrckt
static int iButtonNetwPanel = -1;			// Knopf im Network Panel gedrckt
unsigned char ucQuitFlag = 1;				// 0 beendet die Software
int iTimerDiv = 0;
unsigned char ucChangeValue = 1;
int iWifiBMP[7];							// Handle fr Bitmaps
char sString[1024];						// Generelle Stringverarbeitung

// FTDI
FT_HANDLE FTDIHandle = 0;					// FTDI Handle

// ########## HAUPTPROGRAMM ##########

int main (int argc, char *argv[]) {

	if (InitCVIRTE (0, argv, 0) == 0) { return -1; }

	// Lade Panels
	if ((hdMainPanel = LoadPanel (0, "iClock_WiFi.uir", MAIN_PANEL)) < 0) { return -1; }
	if ((hdDeviPanel = LoadPanel (0, "iClock_WiFi.uir", DEVI_PANEL)) < 0) { return -1; }
	if ((hdNetwPanel = LoadPanel (0, "iClock_WiFi.uir", NETW_PANEL)) < 0) { return -1; }

	// Tabs
	GetPanelHandleFromTabPage (hdMainPanel, MAIN_PANEL_TAB, 0, &hdTab0);
	GetPanelHandleFromTabPage (hdMainPanel, MAIN_PANEL_TAB, 1, &hdTab1);
	GetPanelHandleFromTabPage (hdMainPanel, MAIN_PANEL_TAB, 2, &hdTab2);
	
	// Attribute
	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_HARDWARE_LED, ATTR_CTRL_VAL, 0); 
	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_HARDWARE_LED, ATTR_OFF_COLOR, 0x202020);
	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_HARDWARE_LED, ATTR_ON_COLOR, 0x00F000);
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CONNECT_LED, ATTR_CTRL_VAL, 0); 
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CONNECT_LED, ATTR_OFF_COLOR, 0x202020);
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CONNECT_LED, ATTR_ON_COLOR, 0x00F000);
	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_ID, ATTR_NO_EDIT_TEXT, 1);
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_SSID, ATTR_NO_EDIT_TEXT, 1);
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_HOSTNAME, ATTR_NO_EDIT_TEXT, 1);   
	SetCtrlAttribute (hdNetwPanel, NETW_PANEL_MAC, ATTR_NO_EDIT_TEXT, 1);   

	// Bitmaps
	GetBitmapFromFile ("Images/Connect.bmp", &iWifiBMP[0]);     
	GetBitmapFromFile ("Images/WIFI1.bmp",   &iWifiBMP[1]);
	GetBitmapFromFile ("Images/WIFI2.bmp",   &iWifiBMP[2]);
	GetBitmapFromFile ("Images/WIFI3.bmp",   &iWifiBMP[3]);
	GetBitmapFromFile ("Images/WIFI4.bmp",   &iWifiBMP[4]);
	GetBitmapFromFile ("Images/Open.bmp",    &iWifiBMP[5]);
	GetBitmapFromFile ("Images/Close.bmp",   &iWifiBMP[6]);

	// Panel
	DisplayPanel (hdMainPanel);

	// Quit-Bedingung
	while (ucQuitFlag) {
		Get_Device_Data ();
		ProcessSystemEvents (); 
		Sleep (25.0);
	}

	// Evtl. Gert schlieen
	FTDI_Close_Device ();

	// Panel schlieen
	DiscardPanel (hdMainPanel);
	DiscardPanel (hdDeviPanel);
	DiscardPanel (hdNetwPanel);
	
	return 0;
}	

// ########## CVI CALLBACKS ##########

// Hauptfenster
int CVICALLBACK Main_Panel (int panel, int event, void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_GOT_FOCUS:

			break;
		case EVENT_LOST_FOCUS:

			break;
		case EVENT_CLOSE:
 			ucQuitFlag = 0;
			break;
		}
	return 0;
}

// Device Panel
int CVICALLBACK Devi_Panel (int panel, int event, void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_CLOSE:
 			iButtonDeviPanel = DEVI_PANEL_CANCEL;
			break;
		}
	return 0;
}

// Network Panel
int CVICALLBACK Netw_Panel (int panel, int event, void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_CLOSE:
 			iButtonNetwPanel = NETW_PANEL_CANCEL;
			break;
		}
	return 0;
}

// Knpfe
int CVICALLBACK Buttons (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
	if (event == EVENT_COMMIT) {   
		if (panel == hdDeviPanel) {
			if (control == DEVI_PANEL_SELECT)		{ iButtonDeviPanel = DEVI_PANEL_SELECT; }
			if (control == DEVI_PANEL_LISTBOX)		{ iButtonDeviPanel = DEVI_PANEL_SELECT; }
			if (control == DEVI_PANEL_CANCEL)		{ iButtonDeviPanel = DEVI_PANEL_CANCEL; }
		}
		if (panel == hdNetwPanel) {
			if (control == NETW_PANEL_CANCEL)		{ iButtonNetwPanel = NETW_PANEL_CANCEL; }       
			if (control == NETW_PANEL_SCAN)			{ iButtonNetwPanel = NETW_PANEL_SCAN; }       
			if (control == NETW_PANEL_NTP_SAVE)		{ iButtonNetwPanel = NETW_PANEL_NTP_SAVE; }       
		}
	}
	return 0;
}

// Menuauswahl
void CVICALLBACK Menu_Select (int menuBar, int menuItem, void *callbackData, int panel) {
	switch (menuItem) {
		case MENU_PROJECT_ABOUT:
			MessagePopup ("ber das Programm", "iClock WiFi\n\n(c) 2017 DCR Dirk Conrad\nHollnderstr. 38 - 13407 Berlin - Germany\n+49 (0)151 144 55 627");
			break;
		case MENU_PROJECT_QUIT:
			ucQuitFlag = 0; 
			break;
		}
}

// Hardware finden
int CVICALLBACK Hardware (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
	switch (event) {
		case EVENT_COMMIT:
			if (control == MAIN_PANEL_OPEN_BUTTON) {
				FTDI_Get_Devices ();
			}
			if (control == MAIN_PANEL_CLOSE_BUTTON) {
				FTDI_Close_Device ();   
			}
			break;
		}
	return 0;
}

// Alarm Zeit ndern
int CVICALLBACK Change_Alarm (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
int iValue;
char sTmp[16];

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab1, TAB1_ALARM_H, &iValue);
			Fmt (sTmp, "WAH %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);

			GetCtrlVal (hdTab1, TAB1_ALARM_M, &iValue);
			Fmt (sTmp, "WAM %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);

			GetCtrlVal (hdTab1, TAB1_ALARM_ON_OFF, &iValue);
			Fmt (sTmp, "WAV %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);

			GetCtrlVal (hdTab1, TAB1_ALARM_VOLUME, &iValue);
			Fmt (sTmp, "WVOL %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);

			ucChangeValue = 1;
			break;
		}
	return 0;
}

// Hostname ndern
int CVICALLBACK Change_Host (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sHostName[128];
	
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_HOST_NAME, sString);
			if (strlen (sString) == 0) {
				Fmt (sString, "-");
			}
			Fmt (sHostName, "WHOST %s", sString);
			FTDI_Command (strlen (sHostName) + 9, sString, sHostName);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// URL ndern
int CVICALLBACK Change_URL (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sURLName[128];
	
	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_URL_NAME, sString);
			if (strlen (sString) == 0) {
				Fmt (sString, "/");
			}
			Fmt (sURLName, "WURL %s", sString);
			FTDI_Command (strlen (sURLName) + 9, sString, sURLName);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// Zeitzone ndern
int CVICALLBACK Change_Time_Zone (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sTmp[16];
int iTimeZone;

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_TIME_ZONE, &iTimeZone);
			Fmt (sTmp, "WTZ %d", iTimeZone);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// DST ndern
int CVICALLBACK Change_DST (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sTmp[16];
int iDST;

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_DST, &iDST);
			Fmt (sTmp, "WDST %d", iDST);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// Kalibrierung in ms
int CVICALLBACK Change_Calibration (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sTmp[16];
int iValue;

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_CALIBRATION, &iValue);
			Fmt (sTmp, "WCAL %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// LCD Helligkeit
int CVICALLBACK Change_Brightness (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sTmp[16];
int iValue;

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_BRIGHTNESS, &iValue);
			Fmt (sTmp, "WBRI %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// Animation ndern
int CVICALLBACK Change_Animation (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
char sTmp[16];
int iValue;

	switch (event) {
		case EVENT_COMMIT:
			GetCtrlVal (hdTab2, TAB2_ANIMATION, &iValue);
			Fmt (sTmp, "WLDA %d", iValue);
			FTDI_Command (strlen (sTmp) + 9, sString, sTmp);
			ucChangeValue = 1;
			break;
		}
	return 0;
}

// Netzwerk
int CVICALLBACK Network (int panel, int control, int event, void *callbackData, int eventData1, int eventData2) {
int iRefreshStatus = 0;
int iChange = 0;
int iSkipScan = 0;
int iIP3, iIP2, iIP1, iIP0;   
int iNum;
Point pntCell;
char sSSID[34];
char sPW[34];
char sCommand[64];
char sNTPName[38]; 
char sPIN[21];

	switch (event) {
		case EVENT_COMMIT:
			SetCtrlAttribute (hdMainPanel, MAIN_PANEL_OPEN_BUTTON, ATTR_DIMMED, 1);  
			SetCtrlAttribute (hdMainPanel, MAIN_PANEL_CLOSE_BUTTON, ATTR_DIMMED, 1);  
			SetCtrlAttribute (hdMainPanel, MAIN_PANEL_NETWORK_BUTTON, ATTR_DIMMED, 1);  
			DisplayPanel (hdNetwPanel);
			iButtonNetwPanel = -1;

			while (iButtonNetwPanel != NETW_PANEL_CANCEL) {
				// Cennection Status
				FTDI_Command (-1, sString, "NS");
				Remove_CR (sString);
				SetCtrlVal (hdNetwPanel, NETW_PANEL_NETWORK_STATUS_TEXT, sString);
				if (sString[0] == 'C' && sString[8] == 'D') {
					if (iChange == 0) {
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CONNECT_LED, ATTR_CTRL_VAL, 1);    
						iRefreshStatus = 0;
						iChange = 1;
					}
				}
				else {
					if (iChange == 1) {  
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CONNECT_LED, ATTR_CTRL_VAL, 0);    
						iRefreshStatus = 0; 
						iChange = 0;
					}
				}

				ProcessSystemEvents (); 

				// Nur aktualisieren bei nderung
				if (iRefreshStatus == 0) {
					iRefreshStatus = 1;
					
					// ID
					FTDI_Command (-1, sString, "RID");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_SSID, sString);
					
					// Hostname
					FTDI_Command (-1, sString, "HST");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_HOSTNAME, sString);

					// Local IP
					FTDI_Command (-1, sString, "LIP");
					String_To_IP (sString, &iIP3, &iIP2, &iIP1, &iIP0);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_LIP3, iIP3);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_LIP2, iIP2);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_LIP1, iIP1);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_LIP0, iIP0);

					// Gateway IP
					FTDI_Command (-1, sString, "GIP");
					String_To_IP (sString, &iIP3, &iIP2, &iIP1, &iIP0);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_GIP3, iIP3);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_GIP2, iIP2);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_GIP1, iIP1);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_GIP0, iIP0);

					// Subnet Maske
					FTDI_Command (-1, sString, "SUB");
					String_To_IP (sString, &iIP3, &iIP2, &iIP1, &iIP0);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_SUB3, iIP3);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_SUB2, iIP2);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_SUB1, iIP1);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_SUB0, iIP0);

					// DNS
					FTDI_Command (-1, sString, "DNS");
					String_To_IP (sString, &iIP3, &iIP2, &iIP1, &iIP0);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_DNS3, iIP3);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_DNS2, iIP2);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_DNS1, iIP1);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_DNS0, iIP0);

					// MAC
					FTDI_Command (31, sString, "MAC");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_MAC, sString);
					
					// NTP 1
					FTDI_Command (-1, sString, "RNTP1");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_1, sString);

					// NTP 2
					FTDI_Command (-1, sString, "RNTP2");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_2, sString);

					// NTP 3
					FTDI_Command (-1, sString, "RNTP3");
					Remove_CR (sString);
					SetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_3, sString);

					// Netzwerk Scan
					if (iSkipScan == 0) {
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_SCAN, ATTR_DIMMED, 1); 
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CANCEL, ATTR_DIMMED, 1); 
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_TABLE, ATTR_DIMMED, 1); 
		 				ProcessSystemEvents (); 

						Network_Scan ();
						GetNumTableRows (hdNetwPanel, NETW_PANEL_TABLE, &iNum);
						if (iNum > 0) {
							SetActiveTableCell (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (1, 1));
						}
						iSkipScan = 1;
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_SCAN, ATTR_DIMMED, 0); 
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_CANCEL, ATTR_DIMMED, 0); 
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_TABLE, ATTR_DIMMED, 0); 
						ProcessSystemEvents (); 
					}
				}
				
				// Knpfe im Network Fenster
				if (iButtonNetwPanel == NETW_PANEL_SCAN) {
					iRefreshStatus = 0;
					iButtonNetwPanel = -1;
					iSkipScan = 0; 
				}
				if (iButtonNetwPanel == NETW_PANEL_NTP_SAVE) {
					GetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_1, sString);
					if (strlen (sString) == 0) {
						Fmt (sString, "NO NAME");
					}
					Fmt (sNTPName, "WNTP1 %s", sString);
					FTDI_Command (strlen (sNTPName) + 9, sString, sNTPName);
					GetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_2, sString);
					if (strlen (sString) == 0) {
						Fmt (sString, "NO NAME");
					}
					Fmt (sNTPName, "WNTP2 %s", sString);
					FTDI_Command (strlen (sNTPName) + 9, sString, sNTPName);
					GetCtrlVal (hdNetwPanel, NETW_PANEL_NTP_NAME_3, sString);
					if (strlen (sString) == 0) {
						Fmt (sString, "NO NAME");
					}
					Fmt (sNTPName, "WNTP3 %s", sString);
					FTDI_Command (strlen (sNTPName) + 9, sString, sNTPName);
					iRefreshStatus = 0;
					iButtonNetwPanel = -1;
					iSkipScan = 1; 
				}				

				GetNumTableRows (hdNetwPanel, NETW_PANEL_TABLE, &iNum);
				if (iNum > 0) {
					GetActiveTableCell (hdNetwPanel, NETW_PANEL_TABLE, &pntCell);
					if (pntCell.x == 7 && pntCell.y > 0) {
						SetActiveTableCell (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (1, 1));
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_TABLE, ATTR_VISIBLE, 0);
						ProcessSystemEvents (); 
						GetTableCellVal (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (2, pntCell.y), sSSID);
						Fmt (sCommand, "WID %s", sSSID);
						FTDI_Command (strlen (sCommand) + 9, sString, sCommand);
						Fmt (sPW, "");
						GetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (5, pntCell.y), ATTR_TEXT_BGCOLOR, &iNum);
						if (iNum == 0xFFFFFF) {
							PromptPopup ("PASSWORT", "Bitte das WLAN- Passwort eingeben.", sPW, 32);
						}
						if (strlen (sPW) > 0) {
							Fmt (sCommand, "PW %s", sPW);
						}
						else {
							Fmt (sCommand, "PW");  
							Sleep (500);
						}
						FTDI_Command (strlen (sCommand) + 9, sString, sCommand);
						FTDI_Command (12, sString, "CON");
						iSkipScan = 1;
						SetCtrlAttribute (hdNetwPanel, NETW_PANEL_TABLE, ATTR_VISIBLE, 1);
						ProcessSystemEvents (); 
					}
				}
	
				ProcessSystemEvents (); 
				Sleep (25.0);
			}
			HidePanel (hdNetwPanel);   
			iButtonNetwPanel = -1;   
			SetCtrlAttribute (hdMainPanel, MAIN_PANEL_OPEN_BUTTON, ATTR_DIMMED, 0);  
			if (FTDIHandle) {
				SetCtrlAttribute (hdMainPanel, MAIN_PANEL_CLOSE_BUTTON, ATTR_DIMMED, 0);  
				SetCtrlAttribute (hdMainPanel, MAIN_PANEL_NETWORK_BUTTON, ATTR_DIMMED, 0);  
			}
			break;
		}
	return 0;
}

// ########## UNTERPROGRAMME ########## 

#define MAX_DEVICES			16     

// Get Devices
void FTDI_Get_Devices () {
char *pBufPtrs[MAX_DEVICES + 1];	// Pointer 
char sBuffer[MAX_DEVICES][64];		// Buffer fr Device- Beschreibung 
long lNumOfDevices;
HRESULT iResult = -1;  
int iCnt;
int iDevNum;

	// Gert schlieen
	FTDI_Close_Device ();

	// Pointer initialisieren
	for (iCnt = 0; iCnt < MAX_DEVICES; iCnt++) {
		pBufPtrs[iCnt] = sBuffer[iCnt];
	}    
	// Letzter Pointer = NULL
	pBufPtrs[MAX_DEVICES] = NULL; 

	iResult = FT_ListDevices (pBufPtrs, &lNumOfDevices, FT_LIST_ALL|FT_OPEN_BY_DESCRIPTION); 
	if (iResult) {    
		MessagePopup("Fehler beim Erstellen der Gerteliste", "Kann USB-Gerteliste nicht erstellen!");
		return; 
	}
	ClearListCtrl (hdDeviPanel, DEVI_PANEL_LISTBOX); 
	for (iCnt = 0; iCnt < lNumOfDevices; iCnt++) {
		InsertListItem (hdDeviPanel, DEVI_PANEL_LISTBOX, iCnt, sBuffer[iCnt], iCnt); 
	}

	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_OPEN_BUTTON, ATTR_DIMMED, 1);  
	DisplayPanel (hdDeviPanel);
	while (iButtonDeviPanel != DEVI_PANEL_SELECT && iButtonDeviPanel != DEVI_PANEL_CANCEL) {
		ProcessSystemEvents (); 
		Sleep (50.0);
	}
	if (iButtonDeviPanel == DEVI_PANEL_SELECT) {
		GetNumListItems (hdDeviPanel, DEVI_PANEL_LISTBOX, &iCnt); 
		if (iCnt > 0) {
			GetCtrlIndex (hdDeviPanel, DEVI_PANEL_LISTBOX, &iDevNum);
			FTDI_Open_Device (iDevNum);
			if (FTDIHandle != 0) {
				SetCtrlAttribute (hdMainPanel, MAIN_PANEL_HARDWARE_LED, ATTR_CTRL_VAL, 1);
				SetCtrlAttribute (hdMainPanel, MAIN_PANEL_NETWORK_BUTTON, ATTR_DIMMED, 0);  
				SetCtrlAttribute (hdMainPanel, MAIN_PANEL_CLOSE_BUTTON, ATTR_DIMMED, 0); 
				SetCtrlAttribute (hdTab1, TAB1_ALARM, ATTR_DIMMED, 0); 
				SetCtrlAttribute (hdTab2, TAB2_HOST, ATTR_DIMMED, 0); 
				SetCtrlAttribute (hdTab2, TAB2_URL, ATTR_DIMMED, 0); 
				FTDI_Purge ();
				// Gerte-ID
				FTDI_Command (-1, sString, "ID");
				Remove_CR (sString);
				SetCtrlVal (hdMainPanel, MAIN_PANEL_ID, sString);
			}
		}
	}
	
	HidePanel (hdDeviPanel);
	SetCtrlAttribute (hdMainPanel, MAIN_PANEL_OPEN_BUTTON, ATTR_DIMMED, 0);
	iButtonDeviPanel = -1;
}

// FTDI Open Device
void FTDI_Open_Device (int iDevNum) {
HRESULT iResult = -1;
	// Wenn keine Gertenummer vergeben wurde
	if (iDevNum < 0) {
		return;
	}
	// Wenn bereits geffnet   
	if (FTDIHandle != 0) {
		MessagePopup ("Fehler beim ffnen", "USB-Gert ist bereits geffnet!");
		return;  
	}
	// Gert nach Gertenummer ffnen und den Handle dazu merken    
	iResult = FT_Open(iDevNum, &FTDIHandle);
	if (iResult) {
		MessagePopup("Fehler beim ffnen", "Kann USB-Gert nicht ffnen!");
		FTDIHandle = 0;
		return;
	}
	// Gert zurcksetzen   
	iResult = FT_ResetDevice (FTDIHandle);
	if (iResult != FT_OK) {
		MessagePopup ("Fehler beim Zurcksetzen", "Kann USB-Gert nicht zurcksetzen!");
		FTDIHandle = 0;
		return;
	}
	// Baudrate einstellen
	iResult = FT_SetBaudRate (FTDIHandle, 115200);
	if (iResult) {
		MessagePopup("Fehler beim Konfigurieren", "Kann Baud-Rate nicht setzen!");
		FTDIHandle = 0;
		return;
	}
	// Serielles Protokoll einstellen, (8 Datenbits, 1 Stoppbit, kein Parity)
	iResult = FT_SetDataCharacteristics (FTDIHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE);
	if (iResult) {
		MessagePopup("Fehler beim Konfigurieren", "Kann Data Characteristics nicht setzen!");
		FTDIHandle = 0;
		return;
	}
	// Gre des Transfers einstellen, default bei Seriellen Schnittstellen ist 4096
   	iResult = FT_SetUSBParameters (FTDIHandle, 4096, 4096);
	if (iResult) {
		MessagePopup("Fehler beim Konfigurieren", "Kann USB-Parameter nicht setzen!");
		FTDIHandle = 0;
		return;
	} 
	// Timeouts einstellen, 1. Wert fr Lesen in ms, 2. Wert fr Schreiben in ms
	iResult = FT_SetTimeouts (FTDIHandle, 200, 200);
	if (iResult) {
		MessagePopup("Fehler beim Konfigurieren", "Kann Timeouts nicht setzen");
		FTDIHandle = 0;
		return;
	}
	// Latenz in ms einstellen, Wert sollte 2 nicht unterschreiten
	iResult = FT_SetLatencyTimer (FTDIHandle, 2);
	if (iResult) {
		MessagePopup("Fehler beim Konfigurieren", "Kann Latency Timer nicht setzen!");
		FTDIHandle = 0;
		return;
	}	
}

// FTDI Close Device
void FTDI_Close_Device () {       
HRESULT iResult;

	// Gert muss nur geschlossen werden, wenn es vorher auch offen war
	if (FTDIHandle == 0) { return; } 
	// Gert schlieen
	iResult = FT_Close (FTDIHandle);
	if (iResult != FT_OK) {
		MessagePopup ("Fehler beim Schlieen", "Kann das USB-Gert nicht schlieen!");
	}
	else {
		SetCtrlAttribute (hdMainPanel, MAIN_PANEL_HARDWARE_LED, ATTR_CTRL_VAL, 0);
		SetCtrlAttribute (hdMainPanel, MAIN_PANEL_NETWORK_BUTTON, ATTR_DIMMED, 1);
		SetCtrlAttribute (hdMainPanel, MAIN_PANEL_CLOSE_BUTTON, ATTR_DIMMED, 1);
		SetCtrlAttribute (hdTab1, TAB1_ALARM, ATTR_DIMMED, 1);
		SetCtrlAttribute (hdTab2, TAB2_HOST, ATTR_DIMMED, 1);
		SetCtrlAttribute (hdTab2, TAB2_URL, ATTR_DIMMED, 1);

		SetCtrlVal (hdMainPanel, MAIN_PANEL_ID, "");
		SetCtrlVal (hdTab0, TAB0_TIME_HM, "*");
		SetCtrlVal (hdTab0, TAB0_TIME_SEC, "*");
		SetCtrlVal (hdTab0, TAB0_DATE, "*");
		SetCtrlVal (hdTab0, TAB0_TM, "*");
		SetCtrlVal (hdTab0, TAB0_RH, "*");
		SetCtrlVal (hdTab1, TAB1_ALARM_H, 0);
		SetCtrlVal (hdTab1, TAB1_ALARM_M, 0);
		SetCtrlVal (hdTab1, TAB1_ALARM_ON_OFF, 0);
		SetCtrlVal (hdTab1, TAB1_ALARM_VOLUME, 3);
		SetCtrlVal (hdTab2, TAB2_TIME_ZONE, 0);
		SetCtrlVal (hdTab2, TAB2_DST, 0);
		SetCtrlVal (hdTab2, TAB2_CALIBRATION, 0);
		SetCtrlVal (hdTab2, TAB2_BRIGHTNESS, 1);
		SetCtrlVal (hdTab2, TAB2_ANIMATION, 0);
		SetCtrlVal (hdTab2, TAB2_HOST_NAME, "-");
		SetCtrlVal (hdTab2, TAB2_URL_NAME, "/");
		
		iButtonNetwPanel = NETW_PANEL_CANCEL;
		ucChangeValue = 1;
		
		FTDIHandle = 0;
	}
}

// FTDI Command
void FTDI_Command (int iReadLen, char *sSerReadBuffer, char *sSerWriteBuffer) {
char sSerFindPat[256];		// berprfen auf Gltigkeit der Daten
HRESULT iResult; 
DWORD lpdwAmountInRxQueue;	// Anzahl der Bytes die sich in der Warteschleife befinden
DWORD lpdwReadLen;			// Anzahl der gelesenden Bytes
DWORD dwBytesToWrite;		// Anzahl der Bytes, die zu schreiben sind  
DWORD lpdwBytesWritten;		// Anzahl der Bytes, die wirklich geschrieben wurden
int iCnt, iOffset, iLen;
int iRepeat = 5;
time_t tmr1, tmr2;


	// Gert muss geffnet sein
	if (FTDIHandle == 0) { return; } 
	
	// Suchpattern bauen
	Fmt (sSerFindPat, "CMD:%s:OK", sSerWriteBuffer);

	while (1) {
	
		// Schreiben
		dwBytesToWrite = strlen (sSerWriteBuffer);
		iResult = FT_Write(FTDIHandle, sSerWriteBuffer, dwBytesToWrite, &lpdwBytesWritten); 
		if (iResult || dwBytesToWrite != lpdwBytesWritten) {
			MessagePopup ("FTDI Write", "Kann nicht auf das USB-Gert schreiben!");
			FTDI_Close_Device ();
			return;
		}	
		iResult = FT_Write(FTDIHandle, "\r", 1, &lpdwBytesWritten);
		if (iResult || lpdwBytesWritten != 1) {
			MessagePopup ("FTDI Write", "Kann nicht auf das USB-Gert schreiben!");
			FTDI_Close_Device ();
			return;
		}

		// Wieviel Bytes warten darauf bertragen zu werden?
		time (&tmr1);
		time (&tmr2);
		iLen = 0;
		lpdwAmountInRxQueue = 0;
		while (1) {
			if (iReadLen < 0) {
				Sleep (25);
			}
			iLen = lpdwAmountInRxQueue;
			iResult = FT_GetQueueStatus (FTDIHandle, &lpdwAmountInRxQueue); 
			if (iResult) {
				sSerReadBuffer[0] = 0;
				MessagePopup ("FTDI Read", "Kann Queue Status nicht lesen!");  
				FTDI_Close_Device ();
				return;
			}
			time (&tmr2);
			if (iReadLen == lpdwAmountInRxQueue) {
				break; 
			}
			// Time out nach 6s
			if (tmr2 - tmr1 > 6) {
				break;
			}
			if (iLen == lpdwAmountInRxQueue && iLen > (strlen (sSerFindPat) + 2) && iReadLen < 0) {
				break;
			}
		}
		// Von der seriellen Schnittstelle lesen
		iResult = FT_Read (FTDIHandle, sSerReadBuffer, lpdwAmountInRxQueue, &lpdwReadLen);
		if (iResult) {
			sSerReadBuffer[0] = 0;
			MessagePopup ("FTDI Read", "Kann nicht vom USB-Gert lesen!");
			FTDI_Close_Device ();
			return;
		}
		sSerReadBuffer[lpdwReadLen] = 0;

		// Gltigkeit berprfen
		if (FindPattern (sSerReadBuffer, 0, -1, sSerFindPat, 1, 0) == -1) {
			sSerReadBuffer[0] = 0;
			iRepeat--;
			if (iRepeat < 1) {
				MessagePopup ("FTDI Read", "Fehler beim Lesen - Daten sind ungltig!");  
				FTDI_Close_Device ();
				return;
			}
		}
		else {
			break;
		}
	}

	iOffset = strlen(sSerFindPat) + 2;
	iLen = (strlen(sSerReadBuffer) - iOffset); 
	for (iCnt = 0; iCnt < iLen; iCnt++) {
		sSerReadBuffer[iCnt] = sSerReadBuffer[iCnt + iOffset];
	}
	sSerReadBuffer[iCnt] = 0;
}	

// FTDI Purge
void FTDI_Purge () {
HRESULT iResult;			// Ruckgabestatus     

	iResult = FT_Purge(FTDIHandle, FT_PURGE_RX|FT_PURGE_TX); 
	if (iResult) {
		MessagePopup ("FTDI Purge", "Befehl 'FT Purge' ist fehlgeschlagen!");
		return;
	}		
}

// Daten vom Gert holen
void Get_Device_Data () {
int iValue;
int iCnt;
int iPnt;
char sTmp[32];

	// Wenn kein Gert angeschlossen ist, muss auch nichts geholt werden
	if (FTDIHandle == 0) {
		return; 
	}   

	iTimerDiv--;
	if (iTimerDiv < 1) {
		iTimerDiv = 80;
	}

	if (ucChangeValue) {
		ucChangeValue = 0;

		// Alarm H
		FTDI_Command (-1, sString, "RAH");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab1, TAB1_ALARM_H, iValue);

		// Alarm M
		FTDI_Command (-1, sString, "RAM");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab1, TAB1_ALARM_M, iValue);

		// Alarm AN / AUS
		FTDI_Command (-1, sString, "RAV");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab1, TAB1_ALARM_ON_OFF, iValue);

		// Alarm Volume
		FTDI_Command (-1, sString, "RVOL");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab1, TAB1_ALARM_VOLUME, iValue);

		// Zeitzone
		FTDI_Command (-1, sString, "RTZ");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab2, TAB2_TIME_ZONE, iValue);

		// DST
		FTDI_Command (1 + (11+4), sString, "RDST");
		Remove_CR (sString);
		if (sString[0] == '0') {
			SetCtrlVal (hdTab2, TAB2_DST, 0);
		}
		else {
			SetCtrlVal (hdTab2, TAB2_DST, 1);
		}

		// Calibration
		FTDI_Command (-1, sString, "RCAL");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab2, TAB2_CALIBRATION, iValue);

		// LCD Helligkeit
		FTDI_Command (-1, sString, "RBRI");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab2, TAB2_BRIGHTNESS, iValue);

		// LED Animation
		FTDI_Command (-1, sString, "RLDA");
		Remove_CR (sString);
		Scan (sString, "%d", &iValue); 
		SetCtrlVal (hdTab2, TAB2_ANIMATION, iValue);

		// Hostname
		FTDI_Command (-1, sString, "RHOST");
		Remove_CR (sString);
		SetCtrlVal (hdTab2, TAB2_HOST_NAME, sString);

		// URL Name
		FTDI_Command (-1, sString, "RURL");
		Remove_CR (sString);
		SetCtrlVal (hdTab2, TAB2_URL_NAME, sString);
	}

	if ((iTimerDiv % 20) == 0) {
		// Temperatur
		FTDI_Command (4 + (11+2), sString, "TM");
		Remove_CR (sString);
		Fmt (sString, "%s C", sString);
		SetCtrlVal (hdTab0, TAB0_TM, sString);

		// RH
		FTDI_Command (4 + (11+2), sString, "RH");
		Remove_CR (sString);
		Fmt (sString, "%s%c RH", sString, 37);
		SetCtrlVal (hdTab0, TAB0_RH, sString);
	}

	if ((iTimerDiv % 5) == 0) {    
		// Uhrzeit und Datum
		FTDI_Command (24 + (11+4), sString, "TIME");
		Remove_CR (sString);
		sTmp[0] = sString[0];
		sTmp[1] = sString[1];
		sTmp[2] = sString[2];
		sTmp[3] = sString[3];
		sTmp[4] = sString[4];
		sTmp[5] = 0;
		SetCtrlVal (hdTab0, TAB0_TIME_HM, sTmp);
		sTmp[0] = sString[6];
		sTmp[1] = sString[7];
		sTmp[2] = 0;
		SetCtrlVal (hdTab0, TAB0_TIME_SEC, sTmp);
		iPnt = 0;
		for (iCnt = 0; iCnt < 15; iCnt++) {
			sTmp[iPnt] = sString[iCnt + 9];
			iPnt++;
			if (sString[iCnt + 9] == '.') {
				sTmp[iPnt] = ' ';
				iPnt++;
			}
		}
		sTmp[iPnt] = 0;
		SetCtrlVal (hdTab0, TAB0_DATE, sTmp);  		
	}
}

// CR aus String lschen
void Remove_CR (char *sString) {    
int iCnt;

	for (iCnt = 0; iCnt < strlen (sString); iCnt++) {
		if (sString[iCnt] == '\r' || sString[iCnt] == '\n') {
			sString[iCnt] = '\0';
		}
	}
}

// String zu IP- Zahlen wandeln
void String_To_IP (char *sIP, int *iIP3, int *iIP2, int *iIP1, int *iIP0) {
int iCnt;
int iPnt;
int iPos;
char sTmp[16];

	iPos = 0;
	iCnt = 0;
	iPnt = 0;    
	for (iCnt = iCnt; iCnt < strlen (sIP); iCnt++) {
		if (sIP[iCnt] != '.' && sIP[iCnt] != '\r') { 
			sTmp[iPnt] = sIP[iCnt];
			iPnt++;
		}
		else {
			sTmp[iPnt] = 0;
			iPnt = 0;
			switch (iPos) {
				case 0: Scan (sTmp, "%d", iIP3); break;
				case 1: Scan (sTmp, "%d", iIP2); break;
				case 2: Scan (sTmp, "%d", iIP1); break;
				case 3: Scan (sTmp, "%d", iIP0); break;
			}
			iPos++;
		}
	}
}

// Netzwerk Scan
void Network_Scan () {
int iX, iY;
int iPnt;
int iCnt;
char sTmp[1024];
int iWifiBMPNum;
int iIdx;
double fDB;

	iX = 0;
	iY = 0;
	iCnt = 0;
	iPnt = 0;   
	DeleteTableRows (hdNetwPanel, NETW_PANEL_TABLE, 1, -1);
	ProcessSystemEvents ();
	FTDI_Command (-1, sString, "SCAN");

	for (iCnt = iCnt; iCnt < strlen (sString); iCnt++) {
		if (sString[iCnt] != '|' && sString[iCnt] != '\r' && sString[iCnt] != '\n') { 
			sTmp[iPnt] = sString[iCnt];
			iPnt++;
		}
		else {
			if (sString[iCnt] != '\n') {
				sTmp[iPnt] = 0;
				iPnt = 0;
				switch (iX) {
					case 0:
						InsertTableRows (hdNetwPanel, NETW_PANEL_TABLE, -1, 1, -1);
						SetTableRowAttribute (hdNetwPanel, NETW_PANEL_TABLE, iY+1, ATTR_SIZE_MODE, VAL_USE_EXPLICIT_SIZE); 
						SetTableRowAttribute (hdNetwPanel, NETW_PANEL_TABLE, iY+1, ATTR_ROW_HEIGHT, 26);
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (7, iY+1), ATTR_CTRL_VAL, iWifiBMP[0]);
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, sTmp);   
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_NO_EDIT_TEXT, 1);
						break;
					case 1:
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, sTmp);   
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_NO_EDIT_TEXT, 1);
						break;
					case 2:
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, sTmp);   
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_NO_EDIT_TEXT, 1);
						break;
					case 3:
						Scan (sTmp, "%f", &fDB);
						iWifiBMPNum = ((int)fDB);
						if (iWifiBMPNum < -90) { iIdx = 4; }
						else if (iWifiBMPNum < -77) { iIdx = 3; }
						else if (iWifiBMPNum < -70) { iIdx = 2; }
						else { iIdx = 1; }
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, iWifiBMP[iIdx]);   
						break;
					case 4:
						if (FindPattern (sTmp, 0, -1, "open", 0, 0) != -1) {
							SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_TEXT_BGCOLOR, 0xF8F8F8);
							SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, iWifiBMP[5]);   
						}
						else {
							SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_TEXT_BGCOLOR, 0xFFFFFF);
							SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, iWifiBMP[6]);   
						}
						break;
					case 5:
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_CTRL_VAL, sTmp);   
						SetTableCellAttribute (hdNetwPanel, NETW_PANEL_TABLE, MakePoint (iX+1, iY+1), ATTR_NO_EDIT_TEXT, 1);
						break;
				}
				iX++;
			}
			else {
				iX = 0;
				iY++;
			}
		}
	}
}
